// XmlValidtation (For .NET 1.1)
// Author: Keyvan Nayyeri (www.nayyeri.net)
// Copyright  2006 Keyvan Nayyeri

using System;
using System.Xml;
using System.Xml.Schema;
using System.IO;
using System.Collections;

namespace XMLValidation
{
	/// <summary>
	/// This class gets the address of an XML and
	/// an XSD Schema file and validates XML file against
	/// this Schema and returns an ArrayList of string
	/// values of all errors.
	/// </summary>
	public class Validator
	{
		#region Private members
		private string XSDPath;
		private XmlTextReader Reader;
		private ArrayList Results = new ArrayList();
		#endregion
		#region Constructor
		/// <summary>
		/// Gets the address of XSD Schema file and sets the relative private member.
		/// with its value.
		/// </summary>
		/// <param name="XSD">The physical path of XSD Schema file.</param>
		public Validator(string XSD)
		{
			this.XSDPath = XSD;
		}// XMLValidator
		#endregion
		#region Public methods
		/// <summary>
		/// This public method tries to return all errors
		/// of validating the XML file against XSD Schema.
		/// </summary>
		/// <param name="XMLPath">The physical path of XML file.</param>
		/// <returns>An ArrayList of string values of all errors</returns>
		public ArrayList ValidateIt(string XMLPath)
		{
			ValidatingProcess(XMLPath);

			return this.Results;
		}// ValidateIt
		#endregion
		#region Private methods
		/// <summary>
		/// Creates an XML Schema and adds it to the collection of our schemas.
		/// </summary>
		private void ValidatingProcess(string XMLPath)
		{
			try
			{
				// 1- Read XML file content
				Reader = new XmlTextReader(XMLPath);

				// 2- Read Schema file content
				StreamReader SR = new StreamReader(this.XSDPath);

							
				// 3- Create a new instance of XmlSchema object
				XmlSchema Schema = new XmlSchema();

				// 4- Set XmlSchema object by calling XmlSchema.Read() method
				Schema = XmlSchema.Read(SR, 
					new ValidationEventHandler(ValidatingReader_ValidationEventHandler));

				// 5- Create a new instance of XmlValidationReader object
				XmlValidatingReader ValidatingReader = new XmlValidatingReader(Reader);
				// 6- Set ValidationType for XmlValidationReader object
				ValidatingReader.ValidationType = ValidationType.Schema;
				// 7- Add Schema to XmlValidationReader Schemas collection
				ValidatingReader.Schemas.Add(Schema);

				// 8- Add your ValidationEventHandler address to 
				// XmlValidationReader's ValidationEventHandler
				ValidatingReader.ValidationEventHandler += 
					new ValidationEventHandler(ValidatingReader_ValidationEventHandler);

				// 9- Read XML content in a loop
				while (ValidatingReader.Read())
				{/*Empty loop*/}

			}// try
				//Handle exceptions if you want
			catch (UnauthorizedAccessException AccessEx)
			{
				throw AccessEx;
			}// catch
			catch (Exception Ex)
			{
				throw Ex;
			}// catch
		}// ValidatingProcess
		/// <summary>
		/// This handler simply adds all erros with their line and
		/// position number to the result.
		/// </summary>
		/// <param name="sender">object</param>
		/// <param name="args">ValidationEventArgs</param>
		private void ValidatingReader_ValidationEventHandler(object sender,	
			ValidationEventArgs args)
		{
			// 10- Implement your logic for each validation iteration
			string strTemp;
			strTemp = "Line: " + this.Reader.LineNumber + " - Position: " + 
				this.Reader.LinePosition + " - " + args.Message;

			this.Results.Add(strTemp);
		}// ValidatingReader_ValidationEventHandler
		#endregion
	}
}





